package io.horizontalsystems.bankwallet.modules.market.topplatforms

import io.horizontalsystems.bankwallet.modules.market.SortingField
import io.horizontalsystems.bankwallet.modules.market.TimeDuration
import io.horizontalsystems.bankwallet.modules.market.sortedByDescendingNullLast
import io.horizontalsystems.bankwallet.modules.market.sortedByNullLast
import io.horizontalsystems.core.ICurrencyManager
import io.horizontalsystems.marketkit.MarketKit
import io.horizontalsystems.marketkit.models.TopPlatform
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.rx2.await
import kotlinx.coroutines.withContext

class TopPlatformsRepository(
    private val marketKit: MarketKit,
    private val currencyManager: ICurrencyManager
) {
    private var itemsCache: List<TopPlatform>? = null

    suspend fun get(
        sortingField: SortingField,
        timeDuration: TimeDuration,
        forceRefresh: Boolean,
        limit: Int? = null,
    ) = withContext(Dispatchers.IO) {
        val currentCache = itemsCache

        val items = if (forceRefresh || currentCache == null) {
            marketKit.topPlatformsSingle(currencyManager.baseCurrency.code).await()
        } else {
            currentCache
        }

        itemsCache = items

        val topPlatformsByPeriod = getTopPlatformItems(items, timeDuration)

        topPlatformsByPeriod.sort(sortingField).let { sortedList ->
            limit?.let { sortedList.take(it) } ?: sortedList
        }
    }

    private fun getTopPlatformItems(
        topPlatforms: List<TopPlatform>,
        timeDuration: TimeDuration
    ): List<TopPlatformItem> {
        return topPlatforms.map { platform ->
            val prevRank = when (timeDuration) {
                TimeDuration.OneDay -> platform.rank1D
                TimeDuration.SevenDay -> platform.rank1W
                TimeDuration.ThirtyDay -> platform.rank1M
            }

            val rankDiff = if (prevRank == platform.rank || prevRank == null) {
                null
            } else {
                prevRank - platform.rank
            }

            val marketCapDiff = when (timeDuration) {
                TimeDuration.OneDay -> platform.change1D
                TimeDuration.SevenDay -> platform.change1W
                TimeDuration.ThirtyDay -> platform.change1M
            }

            TopPlatformItem(
                Platform(platform.uid, platform.name),
                platform.rank,
                platform.protocols,
                platform.marketCap,
                rankDiff,
                marketCapDiff
            )
        }
    }

    fun List<TopPlatformItem>.sort(sortingField: SortingField) = when (sortingField) {
        SortingField.HighestCap -> sortedByDescendingNullLast { it.marketCap }
        SortingField.LowestCap -> sortedByNullLast { it.marketCap }
        SortingField.TopGainers -> sortedByDescendingNullLast { it.changeDiff }
        SortingField.TopLosers -> sortedByNullLast { it.changeDiff }
        else -> this
    }

}

